# definition for resourcetype
COLLECTION=1
OBJECT=None

# attributes for resources
DAV_PROPS=['creationdate', 'displayname', 'getcontentlanguage', 'getcontentlength', 'getcontenttype', 'getetag', 'getlastmodified', 'lockdiscovery', 'resourcetype', 'source', 'supportedlock']

# Request classes in propfind
RT_ALLPROP=1
RT_PROPNAME=2
RT_PROP=3

# server mode
DAV_VERSION_1 = {
        'version' : '1',
        'options' : 
        'GET, HEAD, COPY, MOVE, POST, PUT, PROPFIND, PROPPATCH, OPTIONS, MKCOL, DELETE, TRACE, REPORT'
}

DAV_VERSION_2 = {
        'version' : '1,2',
        'options' : 
        DAV_VERSION_1['options'] + ', LOCK, UNLOCK'
}



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from trytond.model import ModelView, ModelSingleton, ModelSQL, fields
from trytond.pyson import Eval, Not, Bool, PYSONEncoder
from trytond.pool import Pool


__all__ = ['GnuHealthSequences','ChagasDUSurvey']


class GnuHealthSequences(ModelSingleton, ModelSQL, ModelView):
    __name__ = 'gnuhealth.sequences'

    chagas_du_survey_sequence = fields.Property(fields.Many2One('ir.sequence',
        'Chagas Survey Sequence', required=True,
        domain=[('code', '=', 'gnuhealth.chagas_du_survey')]))


class ChagasDUSurvey(ModelSQL, ModelView):
    'Chagas DU Entomological Survey'
    __name__ = 'gnuhealth.chagas_du_survey'

    name = fields.Char ('Survey Code', readonly=True)
    du = fields.Many2One('gnuhealth.du', 'DU', help="Domiciliary Unit")
    survey_date = fields.Date('Date', required=True)

    du_status = fields.Selection([
        (None, ''),
        ('initial', 'Initial'),
        ('unchanged', 'Unchanged'),
        ('better', 'Improved'),
        ('worse', 'Worsen'),
        ], 'Status', help="DU status compared to last visit", required=True, sort=False)
    
    # Findings of Triatomines in the DU
    triatomines =  fields.Boolean('Triatomines', help="Check this box if triatomines were found")
    vector = fields.Selection([
        (None, ''),
        ('t_infestans', 'T. infestans'),
        ('t_brasilensis', 'T. brasilensis'),
        ('r_prolixus', 'R. prolixus'),
        ('t_dimidiata', 'T. dimidiata'),
        ('p_megistus', 'P. megistus'),
        ], 'Vector', help="Vector", sort=False)


    nymphs = fields.Boolean ('Nymphs', "Check this box if triatomine nymphs were found")
    t_in_house = fields.Boolean('Domiciliary', help="Check this box if triatomines were found inside the house")
    t_peri = fields.Boolean('Peri-Domiciliary', help="Check this box if triatomines were found in the peridomiciliary area")
    
    # Infrastructure conditions
    dfloor = fields.Boolean('Floor', help="Current floor can host triatomines")
    dwall = fields.Boolean('Walls', help="Wall materials or state can host triatomines")
    droof = fields.Boolean('Roof', help="Roof materials or state can host triatomines")
    dperi = fields.Boolean('Peri-domicilary', help="Peri domiciliary area can host triatomines")
    
    # Preventive measures
    
    bugtraps = fields.Boolean('Bug traps', help="The DU has traps to detect triatomines")
    
    # Chemical controls
    
    du_fumigation = fields.Boolean('Fumigation', help="The DU has been fumigated")
    fumigation_date = fields.Date('Fumigation Date',help="Last Fumigation Date", states={'invisible': Not(Bool(Eval('du_fumigation')))})
    
    du_paint = fields.Boolean ('Insecticide Paint', help="The DU has been treated with insecticide-containing paint")
    paint_date = fields.Date('Paint Date',help="Last Paint Date", states={'invisible': Not(Bool(Eval('du_paint')))})
    
    observations = fields.Text('Observations')
    next_survey_date = fields.Date('Next survey')
    
    @staticmethod
    def default_survey_date():
        return datetime.now()


    @classmethod
    def create(cls, vlist):
        Sequence = Pool().get('ir.sequence')
        Config = Pool().get('gnuhealth.sequences')

        vlist = [x.copy() for x in vlist]
        for values in vlist:
            if not values.get('name'):
                config = Config(1)
                values['name'] = Sequence.get_id(
                config.chagas_du_survey_sequence.id)

        return super(ChagasDUSurvey, cls).create(vlist)





import logging

log = logging.getLogger(__name__)

try:
    import MySQLdb
except ImportError:
    log.info('No SQL support - MySQLdb missing...')
    pass

import sys

class Mconn:
    def connect(self,username,userpasswd,host,port,db):
        try: connection = MySQLdb.connect(host=host, port=int(port), user=username, passwd=userpasswd,db=db)
        except MySQLdb.OperationalError as message:
            log.error("%d:\n%s" % (message[ 0 ], message[ 1 ] ))
            return 0
        else:
            self.db = connection.cursor()

            return 1

    def execute(self,qry):
        if self.db:
            try: res=self.db.execute(qry)
            except MySQLdb.OperationalError as message:
                log.error("Error %d:\n%s" % (message[ 0 ], message[ 1 ] ))
                return 0

            except MySQLdb.ProgrammingError as message:
                log.error("Error %d:\n%s" % (message[ 0 ], message[ 1 ] ))
                return 0

            else:
                log.debug('Query Returned '+str(res)+' results')
                return self.db.fetchall()

    def create_user(self,user,passwd):
        qry="select * from Users where User='%s'"%(user)
        res=self.execute(qry)
        if not res or len(res) ==0:
            qry="insert into Users (User,Pass) values('%s','%s')"%(user,passwd)
            res=self.execute(qry)
        else:
            log.debug("Username already in use")

    def create_table(self):
        qry="""CREATE TABLE `Users` (
                  `uid` int(10) NOT NULL auto_increment,
                  `User` varchar(60) default NULL,
                  `Pass` varchar(60) default NULL,
            `Write` tinyint(1) default '0',
                  PRIMARY KEY  (`uid`)
                ) ENGINE=MyISAM DEFAULT CHARSET=latin1"""
        self.execute(qry)


    def first_run(self,user,passwd):
        res= self.execute('select * from Users')
        if res or type(res)==type(()) :
            pass
        else:
            self.create_table()
            self.create_user(user,passwd)


    def __init__(self,user,password,host,port,db):
        self.db=0
        self.connect(user,password,host,port,db)




from flask import request
from server.common import Resource
from server.resources import SearchRecords
from server.resources import ReadRecord
from server.resources import ValidateRecord
from server.resources import Routing

class ORG_Create(Resource):
    def post(self):
        """Create interaction"""

        return 'Not implemented', 405

class ORG_Search(Resource):
    def get(self):
        """Search interaction"""
        action = SearchRecords(endpoint='organization', request=request)
        return action.records


class ORG_Validate(Resource):
    def post(self, log_id=None):
        '''Validate interaction'''
        action = ValidateRecord(endpoint='organization', record=request.data)
        return action.valid

class ORG_Record(Resource):
    def get(self, log_id):
        '''Read interaction'''
        action = ReadRecord(endpoint='organization', log_id=log_id)
        return action.record

    def put(self, log_id):
        '''Update interaction'''
        return 'Not supported', 405

    def delete(self, log_id):
        '''Delete interaction'''
        return 'Not implemented', 405

class ORG_Version(Resource):
    def get(self, log_id, v_id=None):
        '''Vread interaction'''
        return 'Not supported', 405


routing = Routing('Organization')
routing['create'] = ORG_Create
routing['search'] = ORG_Search
routing['validate'] = ORG_Validate
routing['record'] = ORG_Record
routing['version'] = ORG_Version

__all__=['routing']



from .base_classes import *
from .support_functions import *


class Order(Resource):
    """A request to perform an action.If the element is present, it must
    have either a @value, an @id, or extensionsText - why the order
    was made."""
    subclass = None
    superclass = Resource
    def __init__(self, id=None, extension=None, modifierExtension=None, language=None, text=None, contained=None, identifier=None, date=None, subject=None, source=None, target=None, reasonCodeableConcept=None, reasonResource=None, authority=None, when=None, detail=None):
        self.original_tagname_ = None
        super(Order, self).__init__(id, extension, modifierExtension, language, text, contained, )
        if identifier is None:
            self.identifier = []
        else:
            self.identifier = identifier
        self.date = date
        self.subject = subject
        self.source = source
        self.target = target
        self.reasonCodeableConcept = reasonCodeableConcept
        self.reasonResource = reasonResource
        self.authority = authority
        self.when = when
        if detail is None:
            self.detail = []
        else:
            self.detail = detail
    def factory(*args_, **kwargs_):
        if Order.subclass:
            return Order.subclass(*args_, **kwargs_)
        else:
            return Order(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_identifier(self): return self.identifier
    def set_identifier(self, identifier): self.identifier = identifier
    def add_identifier(self, value): self.identifier.append(value)
    def insert_identifier(self, index, value): self.identifier[index] = value
    def get_date(self): return self.date
    def set_date(self, date): self.date = date
    def get_subject(self): return self.subject
    def set_subject(self, subject): self.subject = subject
    def get_source(self): return self.source
    def set_source(self, source): self.source = source
    def get_target(self): return self.target
    def set_target(self, target): self.target = target
    def get_reasonCodeableConcept(self): return self.reasonCodeableConcept
    def set_reasonCodeableConcept(self, reasonCodeableConcept): self.reasonCodeableConcept = reasonCodeableConcept
    def get_reasonResource(self): return self.reasonResource
    def set_reasonResource(self, reasonResource): self.reasonResource = reasonResource
    def get_authority(self): return self.authority
    def set_authority(self, authority): self.authority = authority
    def get_when(self): return self.when
    def set_when(self, when): self.when = when
    def get_detail(self): return self.detail
    def set_detail(self, detail): self.detail = detail
    def add_detail(self, value): self.detail.append(value)
    def insert_detail(self, index, value): self.detail[index] = value
    def hasContent_(self):
        if (
            self.identifier or
            self.date is not None or
            self.subject is not None or
            self.source is not None or
            self.target is not None or
            self.reasonCodeableConcept is not None or
            self.reasonResource is not None or
            self.authority is not None or
            self.when is not None or
            self.detail or
            super(Order, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Order', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Order')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Order', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Order'):
        super(Order, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Order')
    def exportChildren(self, outfile, level, namespace_='', name_='Order', fromsubclass_=False, pretty_print=True):
        super(Order, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for identifier_ in self.identifier:
            identifier_.export(outfile, level, namespace_, name_='identifier', pretty_print=pretty_print)
        if self.date is not None:
            self.date.export(outfile, level, namespace_, name_='date', pretty_print=pretty_print)
        if self.subject is not None:
            self.subject.export(outfile, level, namespace_, name_='subject', pretty_print=pretty_print)
        if self.source is not None:
            self.source.export(outfile, level, namespace_, name_='source', pretty_print=pretty_print)
        if self.target is not None:
            self.target.export(outfile, level, namespace_, name_='target', pretty_print=pretty_print)
        if self.reasonCodeableConcept is not None:
            self.reasonCodeableConcept.export(outfile, level, namespace_, name_='reasonCodeableConcept', pretty_print=pretty_print)
        if self.reasonResource is not None:
            self.reasonResource.export(outfile, level, namespace_, name_='reasonResource', pretty_print=pretty_print)
        if self.authority is not None:
            self.authority.export(outfile, level, namespace_, name_='authority', pretty_print=pretty_print)
        if self.when is not None:
            self.when.export(outfile, level, namespace_, name_='when', pretty_print=pretty_print)
        for detail_ in self.detail:
            detail_.export(outfile, level, namespace_, name_='detail', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Order', mapping_=None):
        element = super(Order, self).to_etree(parent_element, name_, mapping_)
        for identifier_ in self.identifier:
            identifier_.to_etree(element, name_='identifier', mapping_=mapping_)
        if self.date is not None:
            date_ = self.date
            date_.to_etree(element, name_='date', mapping_=mapping_)
        if self.subject is not None:
            subject_ = self.subject
            subject_.to_etree(element, name_='subject', mapping_=mapping_)
        if self.source is not None:
            source_ = self.source
            source_.to_etree(element, name_='source', mapping_=mapping_)
        if self.target is not None:
            target_ = self.target
            target_.to_etree(element, name_='target', mapping_=mapping_)
        if self.reasonCodeableConcept is not None:
            reasonCodeableConcept_ = self.reasonCodeableConcept
            reasonCodeableConcept_.to_etree(element, name_='reasonCodeableConcept', mapping_=mapping_)
        if self.reasonResource is not None:
            reasonResource_ = self.reasonResource
            reasonResource_.to_etree(element, name_='reasonResource', mapping_=mapping_)
        if self.authority is not None:
            authority_ = self.authority
            authority_.to_etree(element, name_='authority', mapping_=mapping_)
        if self.when is not None:
            when_ = self.when
            when_.to_etree(element, name_='when', mapping_=mapping_)
        for detail_ in self.detail:
            detail_.to_etree(element, name_='detail', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Order, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'identifier':
            obj_ = Identifier.factory()
            obj_.build(child_)
            self.identifier.append(obj_)
            obj_.original_tagname_ = 'identifier'
        elif nodeName_ == 'date':
            obj_ = dateTime.factory()
            obj_.build(child_)
            self.date = obj_
            obj_.original_tagname_ = 'date'
        elif nodeName_ == 'subject':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.subject = obj_
            obj_.original_tagname_ = 'subject'
        elif nodeName_ == 'source':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.source = obj_
            obj_.original_tagname_ = 'source'
        elif nodeName_ == 'target':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.target = obj_
            obj_.original_tagname_ = 'target'
        elif nodeName_ == 'reasonCodeableConcept':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.reasonCodeableConcept = obj_
            obj_.original_tagname_ = 'reasonCodeableConcept'
        elif nodeName_ == 'reasonResource':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.reasonResource = obj_
            obj_.original_tagname_ = 'reasonResource'
        elif nodeName_ == 'authority':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.authority = obj_
            obj_.original_tagname_ = 'authority'
        elif nodeName_ == 'when':
            obj_ = Order_When.factory()
            obj_.build(child_)
            self.when = obj_
            obj_.original_tagname_ = 'when'
        elif nodeName_ == 'detail':
            obj_ = ResourceReference.factory()
            obj_.build(child_)
            self.detail.append(obj_)
            obj_.original_tagname_ = 'detail'
        super(Order, self).buildChildren(child_, node, nodeName_, True)
# end class Order


class Order_When(BackboneElement):
    """A request to perform an action."""
    subclass = None
    superclass = BackboneElement
    def __init__(self, id=None, extension=None, modifierExtension=None, code=None, schedule=None):
        self.original_tagname_ = None
        super(Order_When, self).__init__(id, extension, modifierExtension, )
        self.code = code
        self.schedule = schedule
    def factory(*args_, **kwargs_):
        if Order_When.subclass:
            return Order_When.subclass(*args_, **kwargs_)
        else:
            return Order_When(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_code(self): return self.code
    def set_code(self, code): self.code = code
    def get_schedule(self): return self.schedule
    def set_schedule(self, schedule): self.schedule = schedule
    def hasContent_(self):
        if (
            self.code is not None or
            self.schedule is not None or
            super(Order_When, self).hasContent_()
        ):
            return True
        else:
            return False
    def export(self, outfile, level, namespace_='', name_='Order.When', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.original_tagname_ is not None:
            name_ = self.original_tagname_
        showIndent(outfile, level, pretty_print)
        outfile.write('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(outfile, level, already_processed, namespace_, name_='Order.When')
        if self.hasContent_():
            outfile.write('>%s' % (eol_, ))
            self.exportChildren(outfile, level + 1, namespace_='', name_='Order.When', pretty_print=pretty_print)
            showIndent(outfile, level, pretty_print)
            outfile.write('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            outfile.write('/>%s' % (eol_, ))
    def exportAttributes(self, outfile, level, already_processed, namespace_='', name_='Order.When'):
        super(Order_When, self).exportAttributes(outfile, level, already_processed, namespace_, name_='Order.When')
    def exportChildren(self, outfile, level, namespace_='', name_='Order.When', fromsubclass_=False, pretty_print=True):
        super(Order_When, self).exportChildren(outfile, level, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.code is not None:
            self.code.export(outfile, level, namespace_, name_='code', pretty_print=pretty_print)
        if self.schedule is not None:
            self.schedule.export(outfile, level, namespace_, name_='schedule', pretty_print=pretty_print)
    def to_etree(self, parent_element=None, name_='Order.When', mapping_=None):
        element = super(Order_When, self).to_etree(parent_element, name_, mapping_)
        if self.code is not None:
            code_ = self.code
            code_.to_etree(element, name_='code', mapping_=mapping_)
        if self.schedule is not None:
            schedule_ = self.schedule
            schedule_.to_etree(element, name_='schedule', mapping_=mapping_)
        if mapping_ is not None:
            mapping_[self] = element
        return element
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
        return self
    def buildAttributes(self, node, attrs, already_processed):
        super(Order_When, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'code':
            obj_ = CodeableConcept.factory()
            obj_.build(child_)
            self.code = obj_
            obj_.original_tagname_ = 'code'
        elif nodeName_ == 'schedule':
            obj_ = Schedule.factory()
            obj_.build(child_)
            self.schedule = obj_
            obj_.original_tagname_ = 'schedule'
        super(Order_When, self).buildChildren(child_, node, nodeName_, True)
# end class Order_When



# -*- coding: utf-8 -*-
##############################################################################
#
#    GNU Health: The Free Health and Hospital Information System
#    Copyright (C) 2008-2017 Luis Falcon <lfalcon@gnusolidario.org>
#    Copyright (C) 2011-2017 GNU Solidario <health@gnusolidario.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from trytond.model import ModelView, ModelSQL, fields
from trytond.pool import Pool
from dateutil.relativedelta import relativedelta
from datetime import datetime, timedelta, date
from trytond.pyson import Eval, Equal

__all__ = [    
    'OphthalmologyEvaluation',
    'OphthalmologyFindings',    
    ]



class OphthalmologyEvaluation(ModelSQL, ModelView):
    'Ophthalmology Evaluation'
    __name__ = 'gnuhealth.ophthalmology.evaluation'

    STATES = {'readonly': Eval('state') == 'done'}

    patient = fields.Many2One('gnuhealth.patient', 'Patient', required=True)
    visit_date = fields.DateTime('Date', help="Date of Consultation")
    computed_age = fields.Function(fields.Char(
            'Age',
            help="Computed patient age at the moment of the evaluation"),
            'patient_age_at_evaluation')

    gender = fields.Function(fields.Selection([
        (None, ''),
        ('m', 'Male'),
        ('f', 'Female'),
        ], 'Gender'), 'get_patient_gender', searcher='search_patient_gender')

    health_professional = fields.Many2One(
        'gnuhealth.healthprofessional', 'Health Professional', readonly=True,
        help="Health professional / Ophthalmologist / OptoMetrist"
        )

    # there are two types of charts, a meter chart.. 6/.. val
    # and ft chart.. 200/...
    snellen_chart = [
        (None, ''),
        ('6_6', '6/6'),
        ('6_9', '6/9'),
        ('6_12', '6/12'),
        ('6_18', '6/18'),
        ('6_24', '6/24'),
        ('6_36', '6/36'),
        ('6_60', '6/60'),
        ('5_60', '5/60'),
        ('4_60', '4/60'),
        ('3_60', '3/60'),
        ('2_60', '2/60'),
        ('1_60', '1/60'),
        ('1_meter_fc', '1 Meter FC'),
        ('1_2_meter_fc', '1/2 Meter FC'),
        ('hmfc', 'HMCF'),
        ('p_l', 'P/L'), 
        ]
    
    # Near vision chart
    near_vision_chart = [
        (None, ''),
        ('N6', 'N6'),
        ('N8', 'N8'),
        ('N12', 'N12'),
        ('N18', 'N18'),
        ('N24', 'N24'),
        ('N36', 'N36'),
        ('N60', 'N60'),
        ]
    # vision test using snellen chart
    rdva = fields.Selection(snellen_chart, 'RDVA',
                            help="Right Eye Vision of Patient without aid",
                            sort=False, states = STATES)
    ldva = fields.Selection(snellen_chart, 'LDVA',
                            help="Left Eye Vision of Patient without aid",
                            sort=False, states = STATES)
    # vision test using pinhole accurate manual testing
    rdva_pinhole = fields.Selection(snellen_chart, 'RDVA',
                                    help="Right Eye Vision Using Pin Hole",
                                    sort=False, states = STATES)
    ldva_pinhole = fields.Selection(snellen_chart, 'LDVA',
                                    help="Left Eye Vision Using Pin Hole",
                                    sort=False, states = STATES)
    # vison testing with glasses just to assess what the patient sees with
    # his existing aid # useful esp with vision syndromes that are not
    # happening because of the lens
    rdva_aid = fields.Selection(snellen_chart, 'RDVA AID',
                                help="Vision with glasses or contact lens",
                                sort=False, states = STATES)
    ldva_aid = fields.Selection(snellen_chart, 'LDVA AID',
                                help="Vision with glasses or contact lens",
                                sort=False, states = STATES)

    # spherical
    rspherical = fields.Float('SPH',help='Right Eye Spherical', states = STATES)
    lspherical = fields.Float('SPH',help='Left Eye Spherical', states = STATES)

    # cylinder
    rcylinder = fields.Float('CYL',help='Right Eye Cylinder', states = STATES)
    lcylinder = fields.Float('CYL',help='Left Eye Cylinder', states = STATES)
    
    #axis
    raxis = fields.Float('Axis',help='Right Eye Axis', states = STATES)
    laxis = fields.Float('Axis',help='Left Eye Axis', states = STATES)

    # near vision testing .... you will get it when u cross 40 :)
    # its also thinning of the lens.. the focus falls behind the retina
    # in case of distant vision the focus does not reach retina
    
    rnv_add = fields.Float('NV Add', help='Right Eye Best Corrected NV Add',
        states = STATES)
    lnv_add = fields.Float('NV Add', help='Left Eye Best Corrected NV Add'
        , states = STATES)
    
    rnv = fields.Selection(near_vision_chart, 'RNV',
                           help="Right Eye Near Vision", sort=False,
                            states = STATES)
    lnv = fields.Selection(near_vision_chart, 'LNV',
                           help="Left Eye Near Vision", sort=False,
                            states = STATES)

    # after the above tests the optometrist or doctor comes to a best conclusion
    # best corrected visual acuity
    # the above values are from autorefraction
    # the doctors decision is final
    # and there could be changes in values of cylinder, spherical and axis
    # these values will go into final prescription of glasses or contact lens
    # by default these values should be auto populated 
    # and should be modifiable by an ophthalmologist
    rbcva_spherical = fields.Float('SPH',
        help='Right Eye Best Corrected Spherical', states = STATES)
    lbcva_spherical = fields.Float('SPH',
        help='Left Eye Best Corrected Spherical', states = STATES)

    rbcva_cylinder = fields.Float('CYL',
        help='Right Eye Best Corrected Cylinder', states = STATES)
    lbcva_cylinder = fields.Float('CYL',
        help='Left Eye Best Corrected Cylinder', states = STATES)

    rbcva_axis = fields.Float('Axis', 
        help='Right Eye Best Corrected Axis', states = STATES)
    lbcva_axis = fields.Float('Axis',
        help='Left Eye Best Corrected Axis', states = STATES)

    rbcva = fields.Selection(snellen_chart, 'RBCVA', 
                help="Right Eye Best Corrected VA", sort=False, states = STATES)
    lbcva = fields.Selection(snellen_chart, 'LBCVA', 
                help="Left Eye Best Corrected VA", sort=False, states = STATES)
    
    rbcva_nv_add = fields.Float('BCVA - Add',
        help='Right Eye Best Corrected NV Add', states = STATES)
    lbcva_nv_add = fields.Float('BCVA - Add',
        help='Left Eye Best Corrected NV Add', states = STATES)

    rbcva_nv = fields.Selection(near_vision_chart, 'RBCVANV', 
                help="Right Eye Best Corrected Near Vision", 
                sort=False, states = STATES)
    lbcva_nv = fields.Selection(near_vision_chart, 'LBCVANV' , 
                help="Left Eye Best Corrected Near Vision",
                sort=False, states = STATES)        
        
    #some other tests of the eyes
    #useful for diagnosis of glaucoma a disease that builds up
    #pressure inside the eye and destroy the retina
    #its also called the silent vision stealer
    #intra ocular pressure
    #there are three ways to test iop
    #   SCHIOTZ
    #   NONCONTACT TONOMETRY
    #   GOLDMANN APPLANATION TONOMETRY

    #notes by the ophthalmologist or optometrist
    notes = fields.Text ('Notes', states = STATES)

    #Intraocular Pressure 
    iop_method = fields.Selection([
        (None,''),
        ('nct', 'Non-contact tonometry'),
        ('schiotz', 'Schiotz tonometry'),
        ('goldmann', 'Goldman tonometry'),
        ], 'Method', help='Tonometry / Intraocular pressure reading method',
        states = STATES)
    
    riop = fields.Float('RIOP',digits=(2,1),
        help="Right Intraocular Pressure in mmHg", states = STATES)

    liop = fields.Float('LIOP',digits=(2,1),
        help="Left Intraocular Pressure in mmHg", states = STATES)
        
    findings = fields.One2Many(
        'gnuhealth.ophthalmology.findings', 'name',
        'Findings',  states = STATES)

    state = fields.Selection([
        (None, ''),
        ('in_progress', 'In progress'),
        ('done', 'Done'),
        ], 'State', readonly=True, sort=False)

    signed_by = fields.Many2One(
        'gnuhealth.healthprofessional', 'Signed by', readonly=True,
        states={'invisible': Equal(Eval('state'), 'in_progress')},
        help="Health Professional that finished the patient evaluation")


    def patient_age_at_evaluation(self, name):
        if (self.patient.name.dob and self.visit_date):
            rdelta = relativedelta(self.visit_date.date(),
                self.patient.name.dob)
            years_months_days = str(rdelta.years) + 'y ' \
                + str(rdelta.months) + 'm ' \
                + str(rdelta.days) + 'd'
            return years_months_days
        else:
            return None

    def get_patient_gender(self, name):
        return self.patient.gender

    @classmethod
    def search_patient_gender(cls, name, clause):
        res = []
        value = clause[2]
        res.append(('patient.name.gender', clause[1], value))
        return res

    @fields.depends('rdva')
    def on_change_with_rbcva(self):
        return self.rdva

    @fields.depends('ldva')
    def on_change_with_lbcva(self):
        return self.ldva

    @fields.depends('rcylinder')
    def on_change_with_rbcva_cylinder(self):
        return self.rcylinder

    @fields.depends('lcylinder')
    def on_change_with_lbcva_cylinder(self):
        return self.lcylinder

    @fields.depends('raxis')
    def on_change_with_rbcva_axis(self):
        return self.raxis
        
    @fields.depends('laxis')
    def on_change_with_lbcva_axis(self):
        return self.laxis    
    
    @fields.depends('rspherical')
    def on_change_with_rbcva_spherical(self):
        return self.rspherical

    @fields.depends('lspherical')
    def on_change_with_lbcva_spherical(self):
        return self.lspherical
    
    @fields.depends('rnv_add')
    def on_change_with_rbcva_nv_add(self):
        return self.rnv_add
    
    @fields.depends('lnv_add')
    def on_change_with_lbcva_nv_add(self):
        return self.lnv_add

    @fields.depends('rnv')
    def on_change_with_rbcva_nv(self):
        return self.rnv

    @fields.depends('lnv')
    def on_change_with_lbcva_nv(self):
        return self.lnv

    @staticmethod
    def default_visit_date():
        return datetime.now()

    @staticmethod
    def default_state():
        return 'in_progress'

    @staticmethod
    def default_health_professional():
        pool = Pool()
        HealthProf= pool.get('gnuhealth.healthprofessional')
        health_professional = HealthProf.get_health_professional()
        return health_professional


    # Show the gender and age upon entering the patient 
    # These two are function fields (don't exist at DB level)
    @fields.depends('patient')
    def on_change_patient(self):
        gender=None
        age=''
        self.gender = self.patient.gender
        self.computed_age = self.patient.age


    @classmethod
    @ModelView.button
    def end_evaluation(cls, evaluations):
        
        evaluation_id = evaluations[0]

        # Change the state of the evaluation to "Done"
        HealthProf= Pool().get('gnuhealth.healthprofessional')

        signing_hp = HealthProf.get_health_professional()

        cls.write(evaluations, {
            'state': 'done',
            'signed_by': signing_hp,
            })
        

    @classmethod
    def __setup__(cls):
        super(OphthalmologyEvaluation, cls).__setup__()

        cls._buttons.update({
            'end_evaluation': {'invisible': Equal(Eval('state'), 'done')}
            })


#this class model contains the detailed assesment of patient by an
#ophthalmologist


class OphthalmologyFindings(ModelSQL, ModelView):    #model class
    'Ophthalmology Findings'
    __name__ = 'gnuhealth.ophthalmology.findings'    #model Name
    
    # Findings associated to a particular evaluation
    name = fields.Many2One('gnuhealth.ophthalmology.evaluation',
        'Evaluation', readonly=True)

    # Structure
    structure = [
        (None, ''),
        ('lid', 'Lid'),
        ('ncs', 'Naso-lacrimal system'),
        ('conjuctiva', 'Conjunctiva'),
        ('cornea', 'Cornea'),
        ('anterior_chamber', 'Anterior Chamber'),
        ('iris', 'Iris'),
        ('pupil', 'Pupil'),
        ('lens', 'Lens'),
        ('vitreous', 'Vitreous'),
        ('fundus_disc', 'Fundus Disc'),
        ('macula', 'Macula'),
        ('fundus_background', 'Fundus background'),
        ('fundus_vessels', 'Fundus vessels'),
        ('other', 'Other'),
        ]

    eye_structure = fields.Selection(structure,  
        'Structure',help="Affected eye structure",sort=False)

    affected_eye = fields.Selection([  
            (None,''),
            ("right","right"),
            ("left","left"),
            ("both","both"),
        ],'Eye',help="Affected eye",sort=False)

    finding = fields.Char('Finding')    
